unit DbButtons;

interface

uses
{$ifdef WIN32}
  ComCtrls, CommCtrl,
{$endif}
  WinTypes, WinProcs, Messages, SysUtils, Classes,
  Graphics, Controls, StdCtrls, DB, DbTables, Buttons, Explbtn;

type
  TDBOperators = (dboNone, dboFirst, dboLast, dboNext, dboPrior, dboInsert,
                  dboAppend, dboEdit, dboPost, dboCancel, dboDelete, dboRefresh);
//=============================================================================
  TDBBitBtnK = class(TBitBtn)
  private
    fDataLink: TFieldDataLink;
    fDBOperator: TDBOperators;
    function GetDataSource: TDataSource;
    procedure SetDataSource(DataSource: TDataSource);
    procedure DataChange(Sender: TObject);
    procedure Notification(Component: TComponent; Operation: TOperation);
        override;
  {$ifndef VER80}
    procedure CmGetDataLink(var Msg: TMessage); message Cm_GetDataLink;
  {$endif}
    procedure SetDBOperator(Value: TDBOperators);
  protected
    procedure Click; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DBOperator: TDBOperators read fDBOperator write SetDBOperator
                                                               default dboNone;
  end;
//=============================================================================
  TDBSpeedBtnK = class(TSpeedButton)
  private
    fDataLink: TFieldDataLink;
    fDBOperator: TDBOperators;
    function GetDataSource: TDataSource;
    procedure SetDataSource(DataSource: TDataSource);
    procedure DataChange(Sender: TObject);
    procedure Notification(Component: TComponent; Operation: TOperation);
        override;
  {$ifndef VER80}
    procedure CmGetDataLink(var Msg: TMessage); message Cm_GetDataLink;
  {$endif}
    procedure SetDBOperator(Value: TDBOperators);
  protected
    procedure Click; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DBOperator: TDBOperators read fDBOperator write SetDBOperator
                                                               default dboNone;
  end;
//=============================================================================
  TDBExplorerBtnK = class(TExplorerButton)
  private
    fDataLink: TFieldDataLink;
    fDBOperator: TDBOperators;
    function GetDataSource: TDataSource;
    procedure SetDataSource(DataSource: TDataSource);
    procedure DataChange(Sender: TObject);
    procedure Notification(Component: TComponent; Operation: TOperation);
        override;
  {$ifndef VER80}
    procedure CmGetDataLink(var Msg: TMessage); message Cm_GetDataLink;
  {$endif}
    procedure SetDBOperator(Value: TDBOperators);
  protected
    procedure Click; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DBOperator: TDBOperators read fDBOperator write SetDBOperator
                                                               default dboNone;
  end;

procedure Register;

implementation

constructor TDBBitBtnK.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  fDataLink := TFieldDataLink.Create;
  fDataLink.OnDataChange := DataChange;
  DataChange(Self);
end;

destructor TDBBitBtnK.Destroy;
begin
  //fDataLink.OnDataChange := nil;
  fDataLink.Free;
  inherited Destroy;
end;

{ Refresh the link when DBOperator is changed }
procedure TDBBitBtnK.SetDBOperator(Value: TDBOperators);
begin
   if Value <> FDBOperator then begin
      fDBOperator := Value;
      DataChange(Self);
   end;
end;


{ Return a reference to the data source, or nil. }
function TDBBitBtnK.GetDataSource: TDataSource;
begin
  Result := fDataLink.DataSource
end;

{ Change the data source. In Delphi 2.0, request notification if
  the DataSource component is removed from the form. }
procedure TDBBitBtnK.SetDataSource(DataSource: TDataSource);
begin
  fDataLink.DataSource := DataSource;
{$ifndef VER80}
  { Notify the TDBBitBtnK component if the DataSource object is removed. }
  if DataSource <> nil then
    DataSource.FreeNotification(Self);
{$endif}
end;

{ If the DataSource component is removed from the form, then clear the
  DataSource pointer in the data link. }
procedure TDBBitBtnK.Notification(Component: TComponent; Operation: TOperation);
begin
  inherited Notification(Component, Operation);
  if (fDataLink <> nil) and
     (Component = DataSource) and
     (Operation = opRemove)
  then
    DataSource := nil
end;

{ The record changed, so update the Enabled to match. }
procedure TDBBitBtnK.DataChange(Sender: TObject);
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  if (Datasource = nil) then exit;
  if (Datasource.DataSet = nil) then exit;
  with DataSource.DataSet do begin
    if (Active = false) then begin
      Self.Enabled := false;
      Exit;
    end;
    if (State = dsBrowse) then begin
      case DBOperator of
      dboPrior:
        begin
          if (Recordcount > 0) then begin
            if BOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboNext:
        begin
          if (Recordcount > 0) then begin
            if EOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboDelete, dboEdit, DBOFirst, DBOLast:
        if (Recordcount > 0) then
          Self.Enabled := true
        else
          Self.Enabled := false;
      dboPost, dboCancel:
        Self.Enabled := false;
      dboInsert, dboAppend, dboRefresh:
        Self.Enabled := true;
      end;
    end
    else if ((State = dsEdit) or (State = dsInsert)) then begin
      case DBOperator of
      dboPost, dboCancel:
        Enabled := true;
      else
        Enabled := false;
      end;
    end;
  end;
end;

{$ifndef VER80}
{ In Delphi 2.0, a data aware control must respond to
  the Cm_GetDataLink message by returning its
  data link reference. }
procedure TDBBitBtnK.CmGetDataLink(var Msg: TMessage);
begin
  Msg.Result := LongInt(fDataLink);
end;
{$endif}

procedure TDBBitBtnK.Click;
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  inherited Click;
  if DataSource = nil then Exit;
  with DataSource.DataSet do begin
    case DBOperator of
    DBOFirst:
      First;
    DBOLast:
      Last;
    dboNext:
      Next;
    dboPrior:
      Prior;
    dboInsert:
      Insert;
    dboAppend:
      Append;
    dboEdit:
      Edit;
    dboPost:
      Post;
    dboCancel:
      Cancel;
    dboDelete:
      Delete;
    dboRefresh:
      Refresh;
    end;
  end;
end;

//==============================================================================
constructor TDBSpeedBtnK.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  fDataLink := TFieldDataLink.Create;
  fDataLink.OnDataChange := DataChange;
  DataChange(Self);
end;

destructor TDBSpeedBtnK.Destroy;
begin
  //fDataLink.OnDataChange := nil;
  fDataLink.Free;
  inherited Destroy;
end;

{ Refresh the link when DBOperator is changed }
procedure TDBSpeedBtnK.SetDBOperator(Value: TDBOperators);
begin
   if Value <> FDBOperator then begin
      fDBOperator := Value;
      DataChange(Self);
   end;
end;


{ Return a reference to the data source, or nil. }
function TDBSpeedBtnK.GetDataSource: TDataSource;
begin
  Result := fDataLink.DataSource
end;

{ Change the data source. In Delphi 2.0, request notification if
  the DataSource component is removed from the form. }
procedure TDBSpeedBtnK.SetDataSource(DataSource: TDataSource);
begin
  fDataLink.DataSource := DataSource;
{$ifndef VER80}
  { Notify the TDBSpeedBtnK component if the DataSource object is removed. }
  if DataSource <> nil then
    DataSource.FreeNotification(Self);
{$endif}
end;

{ If the DataSource component is removed from the form, then clear the
  DataSource pointer in the data link. }
procedure TDBSpeedBtnK.Notification(Component: TComponent; Operation: TOperation);
begin
  inherited Notification(Component, Operation);
  if (fDataLink <> nil) and
     (Component = DataSource) and
     (Operation = opRemove)
  then
    DataSource := nil
end;

{ The record changed, so update the Enabled to match. }
procedure TDBSpeedBtnK.DataChange(Sender: TObject);
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  if (Datasource = nil) then exit;
  if (Datasource.DataSet = nil) then exit;
  with DataSource.DataSet do begin
    if (State = dsBrowse) then begin
      case DBOperator of
      dboPrior:
        begin
          if (Recordcount > 0) then begin
            if BOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboNext:
        begin
          if (Recordcount > 0) then begin
            if EOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboDelete, dboEdit, DBOFirst, DBOLast:
        if (Recordcount > 0) then
          Self.Enabled := true
        else
          Self.Enabled := false;
      dboPost, dboCancel:
        Self.Enabled := false;
      dboInsert, dboAppend, dboRefresh:
        Self.Enabled := true;
      end;
    end
    else if ((State = dsEdit) or (State = dsInsert)) then begin
      case DBOperator of
      dboPost, dboCancel:
        Enabled := true;
      else
        Enabled := false;
      end;
    end;
  end;
end;

{$ifndef VER80}
{ In Delphi 2.0, a data aware control must respond to
  the Cm_GetDataLink message by returning its
  data link reference. }
procedure TDBSpeedBtnK.CmGetDataLink(var Msg: TMessage);
begin
  Msg.Result := LongInt(fDataLink);
end;
{$endif}

procedure TDBSpeedBtnK.Click;
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  inherited Click;
  if (Datasource = nil) then exit;
  if (Datasource.DataSet = nil) then exit;
  with DataSource.DataSet do begin
    case DBOperator of
    DBOFirst:
      First;
    DBOLast:
      Last;
    dboNext:
      Next;
    dboPrior:
      Prior;
    dboInsert:
      Insert;
    dboAppend:
      Append;
    dboEdit:
      Edit;
    dboPost:
      Post;
    dboCancel:
      Cancel;
    dboDelete:
      Delete;
    dboRefresh:
      Refresh;
    end;
  end;
end;
//==============================================================================
constructor TDBExplorerBtnK.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  fDataLink := TFieldDataLink.Create;
  fDataLink.OnDataChange := DataChange;
  DataChange(Self);
end;

destructor TDBExplorerBtnK.Destroy;
begin
  //fDataLink.OnDataChange := nil;
  fDataLink.Free;
  inherited Destroy;
end;

{ Refresh the link when DBOperator is changed }
procedure TDBExplorerBtnK.SetDBOperator(Value: TDBOperators);
begin
   if Value <> FDBOperator then begin
      fDBOperator := Value;
      DataChange(Self);
   end;
end;


{ Return a reference to the data source, or nil. }
function TDBExplorerBtnK.GetDataSource: TDataSource;
begin
  Result := fDataLink.DataSource
end;

{ Change the data source. In Delphi 2.0, request notification if
  the DataSource component is removed from the form. }
procedure TDBExplorerBtnK.SetDataSource(DataSource: TDataSource);
begin
  fDataLink.DataSource := DataSource;
{$ifndef VER80}
  { Notify the TDBExplorerBtnK component if the DataSource object is removed. }
  if DataSource <> nil then
    DataSource.FreeNotification(Self);
{$endif}
end;

{ If the DataSource component is removed from the form, then clear the
  DataSource pointer in the data link. }
procedure TDBExplorerBtnK.Notification(Component: TComponent; Operation: TOperation);
begin
  inherited Notification(Component, Operation);
  if (fDataLink <> nil) and
     (Component = DataSource) and
     (Operation = opRemove)
  then
    DataSource := nil
end;

{ The record changed, so update the Enabled to match. }
procedure TDBExplorerBtnK.DataChange(Sender: TObject);
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  if (Datasource = nil) then exit;
  if (Datasource.DataSet = nil) then exit;
  with DataSource.DataSet do begin
    if (State = dsBrowse) then begin
      case DBOperator of
      dboPrior:
        begin
          if (Recordcount > 0) then begin
            if BOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboNext:
        begin
          if (Recordcount > 0) then begin
            if EOF then
              Self.Enabled := false
            else
              Self.Enabled := true;
          end
          else begin
            Self.Enabled := false;
          end;
        end;
      dboDelete, dboEdit, DBOFirst, DBOLast:
        if (Recordcount > 0) then
          Self.Enabled := true
        else
          Self.Enabled := false;
      dboPost, dboCancel:
        Self.Enabled := false;
      dboInsert, dboAppend, dboRefresh:
        Self.Enabled := true;
      end;
    end
    else if ((State = dsEdit) or (State = dsInsert)) then begin
      case DBOperator of
      dboPost, dboCancel:
        Enabled := true;
      else
        Enabled := false;
      end;
    end;
  end;
end;

{$ifndef VER80}
{ In Delphi 2.0, a data aware control must respond to
  the Cm_GetDataLink message by returning its
  data link reference. }
procedure TDBExplorerBtnK.CmGetDataLink(var Msg: TMessage);
begin
  Msg.Result := LongInt(fDataLink);
end;
{$endif}

procedure TDBExplorerBtnK.Click;
begin
  {(DBOFirst, DBOLast, dboNext, dboPrior, dboInsert, dboAppend,
                  dboEdit, dboPost, dboCancel, dboDelete, dboRefresh)}
  inherited Click;
  if (Datasource = nil) then exit;
  if (Datasource.DataSet = nil) then exit;
  with DataSource.DataSet do begin
    case DBOperator of
    DBOFirst:
      First;
    DBOLast:
      Last;
    dboNext:
      Next;
    dboPrior:
      Prior;
    dboInsert:
      Insert;
    dboAppend:
      Append;
    dboEdit:
      Edit;
    dboPost:
      Post;
    dboCancel:
      Cancel;
    dboDelete:
      Delete;
    dboRefresh:
      Refresh;
    end;
  end;
end;


procedure Register;
begin
  RegisterComponents('Samples', [TDBBitBtnK]);
  RegisterComponents('Samples', [TDBSpeedBtnK]);
  RegisterComponents('Samples', [TDBExplorerBtnK]);
end;

end.
